@make(ais)
@aiInfo(topic=<Incompatible record alignment and address clauses>,
number=[AI-00337/04],
classcode=[ra],
statuscode=[WI],
mainref=[13.05@\(03)],
date=[88-09-02])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
!class ramification 87-02-17 (provisional classification)
!status work-item 87-02-17
@end(toplines)
!status received 85-05-10
@topic{!topic Incompatible record alignment and address clauses}
@end(history)

@subheading[!summary 87-12-28 (DRAFT)]

An address clause that is not compatible with a previous record
representation alignment clause can be rejected, interpreted so as to be
compatible, or accepted but considered erroneous.  An implementation may
raise PROGRAM_@|ERROR at the place of an erroneous address clause.

@subheading[!question 87-02-17 (DRAFT)]

What is required of an implementation when the alignment given in a record
representation clause is incompatible with the address specified in an
address clause for an object of the record type?  Is PROGRAM_@|ERROR raised?

Consider:
@begin(example)
@key[type] REC @key[is]
     @key[record]
          C : INTEGER;
     @key[end] @key[record];
@hinge()

@key[for] REC @key[use]
     @key[record]
          @key[at] @key[mod] 2;       -- even address alignment
     @key[end] @key[record];
@hinge()

OBJ : REC;
@key[for] OBJ @key[use] @key[at] 1;         -- raise PROGRAM_@|ERROR?
@end(example)
where it is assumed that the implementation defines type SYSTEM.ADDRESS as an
integer type.

Because the expression given in an address clause need not be static, such an
error cannot be a legality issue.  Presumably an implementation should thus
raise an exception at the point of the address clause.  It seems that
PROGRAM_@|ERROR is probably most appropriate, but it would help to clarify
this.

@subheading[!response 87-03-05 (DRAFT)]

The second sentence of 13.4(4) says:
@begin(quotation)
An implementation may place restrictions on the allowable
alignments.
@end(quotation)
The second sentence of 13.5(3) says:
@begin(quotation)
The conventions that define the interpretation of a value of the
type ADDRESS as an address, as an interrupt level, or whatever it
may be, are implementation-dependent.
@end(quotation)
Thus, an implementation could give (in Appendix F) an interpretation such as:
@begin(quotation)
If an address clause is not compatible with a previous record
representation alignment clause, then the closest address that is
compatible is used instead.
@end(quotation)
In this case, the address clause is never erroneous and no exception is
raised.

Alternatively, an implementation could give (in Appendix F) a restriction and
interpretation such as:
@begin(quotation)
An address clause that is not compatible with a previous record
representation alignment clause is rejected if the
incompatibility is determined at compilation time, and is
otherwise erroneous.
@end(quotation)
In general, since the expression in an address clause need not be static, the
incompatibility is not a legality issue.  Nor is it an error that must be
detected at run time, since the restrictions/interpretations are
implementation-dependent.  Nor is it an incorrect order dependence, since the
type and its representation must be declared before the variable and its
address can be declared.  Hence, if the incompatibility is not optionally
rejected at compilation time or resolved to a compatible address at execution
time, then 1.6 leaves only the possibility that the incompatibility be
erroneous.  If the incompatibility is considered erroneous, then 1.6(10)
allows the implementation to raise PROGRAM_@|ERROR at the place of the
incompatible address clause.

@begin(appendix)
@subheading[!appendix @cbon()88-07-28]@cboff()
@begin(sectionGroup)
@mybar()

!section 13.04            Bevin Brett/Ron Brender 85-05-03          83-00543
!version 1983
!topic Inconsistent record alignment and address clause
@end(sectionGroup)

What is required of an implementation when the alignment  given  in  a
record representation is inconsistent with the address specified in an
address clause for an object of the  record  type?   Is  PROGRAM_ERROR
raised?

Consider:

    type REC is record
        C : INTEGER;
        end record;
    for REC use record
        at mod 2;       -- even address alignment
        end record;

    OBJ : REC;
    for OBJ use at 1;   -- raise PROGRAM_ERROR?

where  it  is   assumed   that   the   implementation   defines   type
SYSTEM.ADDRESS as an integer type.

Because the expression given in an address clause need not be  static,
such   an   error   cannot   be   a  legality  issue.   Presumably  an
implementation should thus raise an exception  at  the  point  of  the
address   clause.   It  seems  that  PROGRAM_ERROR  is  probably  most
appropriate, but it would help to clarify this.

@begin(sectionGroup)
@mybar()

!section 13.04    (00) and 13.05 Henry Dancy/ALSYS  85-06-12         83-00573
!version 1983
!topic Inconsistent record alignment and address clause
@end(sectionGroup)
!reference AI-00337/00

The second sentence of 13.04(04) says:

"An implementation may place restrictions on the allowable alignments".

Such a restriction (given in Appendix F) could be:

"An alignment that is not compatible with a subsequent address clause is
erroneous.", which allows the implementation to raise PROGRAM_ERROR at the
place of the address clause.

The fact that this restriction applies to the alignment clause does not seem
to allow an exception to be raised at the place of the address clause.  As
already pointed out in the commentary, it is not a legality issue.  Neither
is it an incorrect order dependence, so 1.6 leaves only the possibility that
it be erroneous.

This would then be a confirmation.

@cbon()@begin(sectionGroup)
@mybar()

!section 13.05    (03) F.Mazzanti  -  S&M                  88-07-28  83-00999
!version 1983
!topic Incompatible record and address clauses
@end(sectionGroup)
--
It is recognized by AI-00337 that sometimes an address clause can be not
compatible with a previous record representation alignment clause.
Among the other alternatives, in AI-00337/02 is suggested that the address
clause could in this case be considered erroneous.
--
We cannot see, however, why the program execution should be allowed to become
'unpredictable' when such an incompatible address clause is elaborated, since
the implementation is surely able to deal with the clause either giving a
particular interpretation to the clause itself or raising PROGRAM_ERROR.
--
In AI-00337 (version 02) is said that 'this kind of error is not an error that
must be detected at run time, since the restrictions/interpretations are
implementation-dependent'; but are not these also the conditions under which
STORAGE_ERROR, NUMERIC_ERROR, DATA_ERROR, NAME_ERROR etc. etc. are raised ?
--
If a new explicit rule allowing PROGRAM_ERROR to be raised is needed, such rule
should be added to the Standard; saying that the address clause can be
erroneous, is not a way to justify that PROGRAM_ERROR is raised, but a way to
justify that PROGRAM_ERROR is not raised and that the elaboration of the
clause causes some compiler or system error.
--
Acknowledge-To: franco <SMPISA@ICNUCEVM>@cboff()

@end(appendix)
